@@ -3,15 +3,16 @@ from numpy import random, dot, array
 
 class PyFBU(object):
     """A class to perform a MCMC sampling.
-
+    
     [more detailed description should be added here]
-
+    
     All configurable parameters are set to some default value, which
     can be changed later on, but before calling the `run` method.
     """
     #__________________________________________________________
-    def __init__(self,data=[],response=[],background={},backgroundsyst={},objsyst={},
-                 lower=[],upper=[],
+    def __init__(self,data=[],response=[],background={},
+                 backgroundsyst={},objsyst={'signal':{},'background':{}},
+                 lower=[],upper=[],regularization=None,
                  rndseed=-1,verbose=False,name='',monitoring=False):
         #                                     [MCMC parameters]
         self.nMCMC = 100000 # N of sampling points    
@@ -22,8 +23,7 @@ class PyFBU(object):
         #                                     [unfolding model parameters]
         self.prior = 'DiscreteUniform'
         self.priorparams = {}
-        self.potential = ''
-        self.potentialparams = {}
+        self.regularization = regularization
         #                                     [input]
         self.data        = data           # data list
         self.response    = response       # response matrix
@@ -36,15 +36,17 @@ class PyFBU(object):
         self.verbose   = verbose
         self.name      = name
         self.monitoring = monitoring
-    
+        
     #__________________________________________________________
     def validateinput(self):
         def checklen(list1,list2):
             assert len(list1)==len(list2), 'Input Validation Error: inconstistent size of input'
-        response = [self.response]+[row for row in self.response]
-        for list in [self.lower,self.upper]+self.background.values()+response:
+        responsetruthbins = self.response
+        responserecobins = [row for row in self.response]
+        for list in self.background.values()+responserecobins:
             checklen(self.data,list)
-        
+        for list in [self.lower,self.upper]:
+            checklen(list,responsetruthbins)
     #__________________________________________________________
     def fluctuate(self, data):
         random.seed(self.rndseed)
@@ -54,55 +56,62 @@ class PyFBU(object):
         self.validateinput()
         data = self.data
         data = self.fluctuate(data) if self.rndseed>=0 else data
-        backgrounds = [array(self.background[syst]) for syst in self.backgroundsyst]
-        backgroundsysts = array(self.backgroundsyst.values())
-        objsysts = array(self.objsyst.values())
-        ndim = len(data)
+
+        # unpack background dictionaries
+        backgroundkeys = self.backgroundsyst.keys()
+        backgrounds = array([self.background[key] for key in backgroundkeys])
+        backgroundnormsysts = array([self.backgroundsyst[key] for key in backgroundkeys])
+
+        # unpack object systematics dictionary
+        objsystkeys = self.objsyst['signal'].keys()
+        signalobjsysts = array([self.objsyst['signal'][key] for key in objsystkeys])
+        backgroundobjsysts = array([])
+        if len(objsystkeys)>0 and len(backgroundkeys)>0:
+            backgroundobjsysts = array([[self.objsyst['background'][syst][bckg] 
+                                         for syst in objsystkeys] 
+                                        for bckg in backgroundkeys])
+        recodim  = len(data)
         resmat = self.response
+        truthdim = len(resmat)
 
         import priors
         truth = priors.wrapper(priorname=self.prior,
                                     low=self.lower,up=self.upper,
                                     other_args=self.priorparams)
 
-        bckgnuisances = [ mc.Normal('gaus_%s'%name,value=self.systfixsigma if err>0.0 else 0.0,
+        bckgnuisances = [ mc.Normal('gaus_%s'%name,value=0.,
                                     mu=0.,tau=1.0,
-                                    observed=(True if (not err>0.0 or self.systfixsigma!=0) 
-                                              else False) )
-                          for name,err in self.backgroundsyst.items() ]
+                                    observed=(False if err>0.0 else True) )
+                          for name,err in zip(backgroundkeys,backgroundnormsysts) ]
         bckgnuisances = mc.Container(bckgnuisances)
         
         objnuisances = [ mc.Normal('gaus_%s'%name,value=self.systfixsigma,mu=0.,tau=1.0,
                                    observed=(True if self.systfixsigma!=0 else False) )
-                         for name,err in self.objsyst.items()]
+                         for name in objsystkeys]
         objnuisances = mc.Container(objnuisances)
 
         # define potential to constrain truth spectrum
-        import potentials
-        if self.potential in potentials.potentialdict:
-            @mc.potential
-            def truthpot(truth=truth):
-                return potentials.wrapper(self.potential,
-                                          truth,size=ndim,
-                                          other_args=self.potentialparams)
-        else:
-            print 'WARNING: potential name not found! Falling back to no potential...'
-            @mc.potential
-            def truthpot():
-                return 0.
+        if self.regularization:
+            truthpot = self.regularization.getpotential(truth)
         
         #This is where the FBU method is actually implemented
         @mc.deterministic(plot=False)
         def unfold(truth=truth,bckgnuisances=bckgnuisances,objnuisances=objnuisances):
-            bckg = dot(1. + bckgnuisances*backgroundsysts,backgrounds)
+            smearbckg = 1.
+            if len(backgroundobjsysts)>0:
+                smearbckg = smearbckg + dot(objnuisances,backgroundobjsysts) 
+            smearedbackgrounds = backgrounds*smearbckg
+            bckg = dot(1. + bckgnuisances*backgroundnormsysts,smearedbackgrounds)
             reco = dot(truth, resmat)
-            smear = 1. + dot(objnuisances,objsysts)
-            out = (bckg + reco)*smear
+            smear = 1. + dot(objnuisances,signalobjsysts)
+            out = bckg + reco*smear
             return out
 
-        unfolded = mc.Poisson('unfolded', mu=unfold, value=data, observed=True, size=ndim)
+        unfolded = mc.Poisson('unfolded', mu=unfold, value=data, observed=True, size=recodim)
         allnuisances = mc.Container(bckgnuisances + objnuisances)
-        model = mc.Model([unfolded, unfold, truth, truthpot, allnuisances])
+        modelelements = [unfolded, unfold, truth, allnuisances]
+        if self.regularization: modelelements += [truthpot]
+        model = mc.Model(modelelements)
 
         map_ = mc.MAP( model ) # this call determines good initial MCMC values
         map_.fit()
@@ -119,12 +128,12 @@ class PyFBU(object):
         mcmc.use_step_method(mc.AdaptiveMetropolis,truth+allnuisances)
         mcmc.sample(self.nMCMC,burn=self.nBurn,thin=self.nThin)
         self.stats = mcmc.stats()
-        self.trace = [mcmc.trace('truth%d'%bin)[:] for bin in xrange(ndim)]
+        self.trace = [mcmc.trace('truth%d'%bin)[:] for bin in xrange(truthdim)]
         self.nuisancestrace = {}
-        for name,err in self.backgroundsyst.items():
+        for name,err in zip(backgroundkeys,backgroundnormsysts):
             if err>0.:
                 self.nuisancestrace[name] = mcmc.trace('gaus_%s'%name)[:]
-        for name in self.objsyst.keys():
+        for name in objsystkeys:
             if self.systfixsigma==0.:
                 self.nuisancestrace[name] = mcmc.trace('gaus_%s'%name)[:]
         
